@@ -22,6 +22,7 @@ import astropy
 from . wcsimage import RefCatalog, WCSImageCatalog, WCSGroupCatalog
 from . tpwcs import TPWCS
 from . matchutils import TPMatch
+from . linearfit import SUPPORTED_FITGEOM_MODES, _SUPPORTED_FITGEOM_EN_STR
 
 from . import __version__
 
@@ -34,7 +35,7 @@ log = logging.getLogger(__name__)
 log.setLevel(logging.DEBUG)
 
 
-def fit_wcs(refcat, imcat, tpwcs, fitgeom='general', nclip=3,
+def fit_wcs(refcat, imcat, tpwcs, ref_tpwcs=None, fitgeom='general', nclip=3,
             sigma=(3.0, 'rmse')):
     """ "Tweak" **a single** image's ``WCS`` by fitting image catalog to a
     reference catalog. This is a simplified version of `align_wcs` that does
@@ -74,7 +75,13 @@ def fit_wcs(refcat, imcat, tpwcs, fitgeom='general', nclip=3,
         a tangent plane that will be used for fitting the two catalogs'
         sources and in which WCS corrections will be applied.
 
-    fitgeom: {'shift', 'rscale', 'general'}, optional
+    ref_tpwcs: TPWCS, None, optional
+        A reference WCS of the type ``TPWCS`` that provides the tangent
+        plane in which matching will be performed and corrections will be
+        defined. When not provided (i.e., set to `None`), reference tangent
+        plane will be the same as defined by ``tpwcs`` argument.
+
+    fitgeom: {'shift', 'rshift', 'rscale', 'general'}, optional
         The fitting geometry to be used in fitting the matched object lists.
         This parameter is used in fitting the offsets, rotations and/or scale
         changes from the matched object lists. The 'general' fit geometry
@@ -229,17 +236,25 @@ def fit_wcs(refcat, imcat, tpwcs, fitgeom='general', nclip=3,
 
     # check fitgeom:
     fitgeom = fitgeom.lower()
-    if fitgeom not in ['shift', 'rscale', 'general']:
-        raise ValueError("Unsupported 'fitgeom'. Valid values are: "
-                         "'shift', 'rscale', or 'general'")
+    if fitgeom not in SUPPORTED_FITGEOM_MODES:
+        raise ValueError(
+            f"Unsupported 'fitgeom'. Valid values are: {_SUPPORTED_FITGEOM_EN_STR:s}"
+        )
 
     wimcat = WCSImageCatalog(imcat, tpwcs,
                              name=imcat.meta.get('name', 'Unnamed'))
     wgcat = WCSGroupCatalog(wimcat, name=imcat.meta.get('name', 'Unnamed'))
     wrefcat = RefCatalog(refcat, name=imcat.meta.get('name', 'Unnamed'))
 
-    succes = wgcat.align_to_ref(refcat=wrefcat, match=None, minobj=None,
-                                fitgeom=fitgeom, nclip=nclip, sigma=sigma)
+    succes = wgcat.align_to_ref(
+        refcat=wrefcat,
+        ref_tpwcs=ref_tpwcs,
+        match=None,
+        minobj=None,
+        fitgeom=fitgeom,
+        nclip=nclip,
+        sigma=sigma
+    )
 
     tpwcs.meta['fit_info'] = wimcat.fit_info
     if not succes:
@@ -257,7 +272,7 @@ def fit_wcs(refcat, imcat, tpwcs, fitgeom='general', nclip=3,
     return wgcat[0].tpwcs
 
 
-def align_wcs(wcscat, refcat=None, enforce_user_order=True,
+def align_wcs(wcscat, refcat=None, ref_tpwcs=None, enforce_user_order=True,
               expand_refcat=False, minobj=None, match=TPMatch(),
               fitgeom='general', nclip=3, sigma=(3.0, 'rmse')):
     r"""
@@ -332,6 +347,14 @@ def align_wcs(wcscat, refcat=None, enforce_user_order=True,
         the ``'weight'`` column, which when present, will be used in fitting.
         See ``Notes`` section for further details.
 
+    ref_tpwcs: TPWCS, None, optional
+        A reference WCS of the type ``TPWCS`` that provides the tangent
+        plane in which matching will be performed and corrections will be
+        defined. When not provided (i.e., set to `None`), reference tangent
+        plane will be defined from the first ``TPWCS`` object
+        *in the re-ordered* (if ``enforce_user_order`` was
+        set to `True`) input list ``wcscat``.
+
     enforce_user_order: bool, optional
         Specifies whether images should be aligned in the order specified in
         the `file` input parameter or `align` should optimize the order
@@ -368,7 +391,7 @@ def align_wcs(wcscat, refcat=None, enforce_user_order=True,
         ``'TPy'`` that represent the source coordinates in some common
         (to both catalogs) coordinate system.
 
-    fitgeom: {'shift', 'rscale', 'general'}, optional
+    fitgeom: {'shift', 'rshift', 'rscale', 'general'}, optional
         The fitting geometry to be used in fitting the matched object lists.
         This parameter is used in fitting the offsets, rotations and/or scale
         changes from the matched object lists. The 'general' fit geometry
@@ -500,17 +523,12 @@ def align_wcs(wcscat, refcat=None, enforce_user_order=True,
 
     # check fitgeom:
     fitgeom = fitgeom.lower()
-    if fitgeom not in ['shift', 'rscale', 'general']:
-        raise ValueError("Unsupported 'fitgeom'. Valid values are: "
-                         "'shift', 'rscale', or 'general'")
-
-    if minobj is None:  # pragma: no branch
-        if fitgeom == 'general':
-            minobj = 3
-        elif fitgeom == 'rscale':
-            minobj = 2
-        else:
-            minobj = 1
+    try:
+        minobj = SUPPORTED_FITGEOM_MODES[fitgeom]
+    except KeyError:
+        raise ValueError(
+            f"Unsupported 'fitgeom'. Valid values are: {_SUPPORTED_FITGEOM_EN_STR:s}"
+        )
 
     # process reference catalog or image if provided:
     if refcat is not None:
@@ -613,6 +631,7 @@ def align_wcs(wcscat, refcat=None, enforce_user_order=True,
 
         current_wcat.align_to_ref(
             refcat=refcat,
+            ref_tpwcs=ref_tpwcs,
             match=match,
             minobj=minobj,
             fitgeom=fitgeom,
@@ -680,7 +699,7 @@ def overlap_matrix(images):
     m = np.zeros((nimg, nimg), dtype=np.double)
     for i in range(nimg):
         for j in range(i + 1, nimg):
-            area = images[i].intersection_area(images[j])
+            area = images[i]._guarded_intersection_area(images[j])
             m[j, i] = area
             m[i, j] = area
     return m
