@@ -54,26 +54,25 @@ def _tp2tp(tpwcs1, tpwcs2, s=None):
     x = np.array([-0.5, 0.5, -0.5, 0.0], dtype=np.double)
     y = np.array([-0.5, -0.5, 0.5, 0.0], dtype=np.double)
 
-    if 'fit_info' in tpwcs1.meta:
-        center = np.array(tpwcs1.meta['fit_info']['center'])
-    else:
-        if tpwcs2.wcs.pixel_bounds is None:
-            # TODO: A pissible improvement would be to get an estimate
-            #       of "center" (where scale is estimated) from source
-            #       positions (if any).
-            center = np.zeros(2)
-        else:
-            center = np.mean(tpwcs2.wcs.pixel_bounds, axis=1)
-
     if s is None:
-        xt, yt = tpwcs1.world_to_tanp(*tpwcs2.det_to_world(center[0] + x, center[1] + y))
-        m = np.array([(xt[1:-1] - xt[0]), (yt[1:-1] - yt[0])])
-        s = np.sqrt(np.fabs(np.linalg.det(m)))
+        if 'fit_info' in tpwcs1.meta and 'center' in tpwcs1.meta['fit_info']:
+            cx, cy = tpwcs1.meta['fit_info']['center']
+            s = tpwcs1.tanp_pixel_scale(cx, cy)
+        else:
+            if tpwcs2.wcs.pixel_bounds is None:
+                # TODO: A possible improvement would be to get an estimate
+                #       of "center" (where scale is estimated) from source
+                #       positions (if any).
+                xt, yt = tpwcs1.world_to_tanp(*tpwcs2.det_to_world(x, y))
+            else:
+                cx, cy = np.mean(tpwcs2.wcs.pixel_bounds, axis=1)
+                xt, yt = tpwcs1.world_to_tanp(
+                    *tpwcs2.det_to_world(cx + x, cy + y)
+                )
+            m = np.array([(xt[1:-1] - xt[0]), (yt[1:-1] - yt[0])])
+            s = np.sqrt(np.fabs(np.linalg.det(m)))
 
-    x *= s
-    y *= s
-
-    xrp, yrp = tpwcs2.world_to_tanp(*tpwcs1.tanp_to_world(center[0] + x, center[1] + y))
+    xrp, yrp = tpwcs2.world_to_tanp(*tpwcs1.tanp_to_world(x * s, y * s))
 
     xrp = np.array(xrp, np.longdouble)
     yrp = np.array(yrp, np.longdouble)
@@ -582,7 +581,6 @@ class JWSTgWCS(TPWCS):
 
             This dictionary **must contain** the following keys and values:
 
-
             'v2_ref': float
                 V2 position of the reference point in arc seconds.
 
@@ -629,7 +627,7 @@ class JWSTgWCS(TPWCS):
         if 'v2v3corr' in frms:
             self._v23name = 'v2v3corr'
             self._tpcorr = deepcopy(
-                wcs.pipeline[frms.index('v2v3corr') - 1][1]
+                wcs.pipeline[frms.index('v2v3corr') - 1].transform
             )
             self._default_tpcorr = None
             if not JWSTgWCS._check_tpcorr_structure(self._tpcorr):
@@ -654,7 +652,7 @@ class JWSTgWCS(TPWCS):
             )
 
         else:
-            self._v23name = 'v2v3'
+            self._v23name = 'v2v3vacorr' if 'v2v3vacorr' in frms else 'v2v3'
             self._tpcorr = None
             self._default_tpcorr = JWSTgWCS._tpcorr_init(
                 v2_ref=v2_ref / 3600.0,
@@ -677,8 +675,8 @@ class JWSTgWCS(TPWCS):
     def _update_transformations(self):
         # define transformations from detector/world coordinates to
         # the tangent plane:
-        detname = self._wcs.pipeline[0][0].name
-        worldname = self._wcs.pipeline[-1][0].name
+        detname = self._wcs.pipeline[0].frame.name
+        worldname = self._wcs.pipeline[-1].frame.name
 
         # Generally needed transformations:
         self._world_to_v23 = self._wcs.get_transform(worldname, self._v23name)
@@ -769,8 +767,21 @@ class JWSTgWCS(TPWCS):
     def _v2v3_to_tpcorr_from_full(tpcorr):
         s2c = tpcorr['s2c']
         c2s = tpcorr['c2s']
-        unit_conv = _get_submodel(tpcorr, 'arcsec_to_deg_2D')
-        unit_conv_inv = _get_submodel(tpcorr, 'deg_to_arcsec_2D')
+
+        # unit_conv = _get_submodel(tpcorr, 'arcsec_to_deg_2D')
+        # unit_conv_inv = _get_submodel(tpcorr, 'deg_to_arcsec_2D')
+        #
+        # The code below is a work-around to the code above not working.
+        # TODO: understand why _get_submodel is unable to retrieve
+        #       some submodels in a compound model.
+        #
+        unit_conv = _get_submodel(tpcorr, 'arcsec_to_deg_1D')
+        unit_conv = unit_conv & unit_conv
+        unit_conv.name = 'arcsec_to_deg_2D'
+        unit_conv_inv = _get_submodel(tpcorr, 'deg_to_arcsec_1D')
+        unit_conv_inv = unit_conv_inv & unit_conv_inv
+        unit_conv_inv.name = 'deg_to_arcsec_2D'
+
         affine = tpcorr['tp_affine']
         affine_inv = affine.inverse
         affine_inv.name = 'tp_affine_inv'
@@ -779,8 +790,20 @@ class JWSTgWCS(TPWCS):
         rot_inv = rot.inverse
         rot_inv.name = 'optic_axis_to_det'
 
-        c2tan = _get_submodel(tpcorr, 'Cartesian 3D to TAN')
-        tan2c = _get_submodel(tpcorr, 'TAN to cartesian 3D')
+        # c2tan = _get_submodel(tpcorr, 'Cartesian 3D to TAN')
+        # tan2c = _get_submodel(tpcorr, 'TAN to cartesian 3D')
+        #
+        # The code below is a work-around to the code above not working.
+        # TODO: understand why _get_submodel is unable to retrieve
+        #       some submodels in a compound model.
+        #
+        c2tan = ((Mapping((0, 1, 2), name='xyz') /
+                  Mapping((0, 0, 0), n_inputs=3, name='xxx')) |
+                 Mapping((1, 2), name='xtyt'))
+        c2tan.name = 'Cartesian 3D to TAN'
+        tan2c = (Mapping((0, 0, 1), n_inputs=2, name='xtyt2xyz') |
+                 (Const1D(1, name='one') & Identity(2, name='I(2D)')))
+        tan2c.name = 'TAN to cartesian 3D'
 
         v2v3_to_tpcorr = unit_conv | s2c | rot | c2tan | affine
         v2v3_to_tpcorr.name = 'jwst_v2v3_to_tpcorr'
@@ -874,32 +897,17 @@ class JWSTgWCS(TPWCS):
         else:
             # combine old and new corrections into a single one and replace
             # old transformation with the combined correction transformation:
-            v2ref, v3ref, roll = self._tpcorr['det_to_optic_axis'].angles.value
-
-            tpcorr2 = JWSTgWCS._tpcorr_init(
-                v2_ref=v2ref,
-                v3_ref=v3ref,
-                roll_ref=roll
-            )
-
             JWSTgWCS._tpcorr_combine_affines(
-                tpcorr2,
+                self._tpcorr,
                 matrix,
                 _ARCSEC2RAD * np.asarray(shift)
             )
 
-            JWSTgWCS._tpcorr_combine_affines(
-                tpcorr2,
-                self._tpcorr['tp_affine'].matrix.value,
-                self._tpcorr['tp_affine'].translation.value
-            )
-
-            self._tpcorr = tpcorr2
-            self._partial_tpcorr = JWSTgWCS._v2v3_to_tpcorr_from_full(tpcorr2)
+            self._partial_tpcorr = JWSTgWCS._v2v3_to_tpcorr_from_full(self._tpcorr)
 
             idx_v2v3 = frms.index(self._v23name)
             pipeline = deepcopy(self._wcs.pipeline)
-            pipeline[idx_v2v3 - 1] = (pipeline[idx_v2v3 - 1][0],
+            pipeline[idx_v2v3 - 1] = (pipeline[idx_v2v3 - 1].frame,
                                       deepcopy(self._tpcorr))
             self._wcs = gwcs.WCS(pipeline, name=self._owcs.name)
 
@@ -924,15 +932,24 @@ class JWSTgWCS(TPWCS):
             return (False, "First and last frames in the WCS pipeline must "
                     "be unique.")
 
-        if frms.count('v2v3') != 1:
-            return (False, "Only one 'v2v3' frame is allowed in the WCS "
-                    "pipeline.")
+        if frms.count('v2v3') != 1 or frms.count('v2v3vacorr') > 1:
+            return (False, "Only one 'v2v3' or 'v2v3vacorr' frame is allowed "
+                    "in the WCS pipeline.")
 
         idx_v2v3 = frms.index('v2v3')
         if idx_v2v3 == 0 or idx_v2v3 == (nframes - 1):
             return (False, "'v2v3' frame cannot be first or last in the WCS "
                     "pipeline.")
 
+        try:
+            idx_v2v3vacorr = frms.index('v2v3vacorr')
+            if idx_v2v3vacorr < idx_v2v3 or idx_v2v3vacorr == (nframes - 1):
+                return (False, "'v2v3vacorr' frame cannot precede the 'v2v3' "
+                        "frame or be the last frame in the WCS pipeline.")
+            idx_v2v3 = idx_v2v3vacorr
+        except ValueError:
+            pass
+
         nv2v3corr = frms.count('v2v3corr')
         if nv2v3corr == 0:
             return True, ''
