@@ -16,7 +16,10 @@ from distutils.version import LooseVersion
 
 import numpy as np
 from astropy import table
-from spherical_geometry.polygon import SphericalPolygon
+from astropy.utils.decorators import deprecated_renamed_argument
+from spherical_geometry.polygon import SphericalPolygon, MalformedPolygonError
+
+from . linearfit import SUPPORTED_FITGEOM_MODES
 
 try:
     import gwcs
@@ -337,6 +340,31 @@ class WCSImageCatalog(object):
                 )
             return area
 
+    def _guarded_intersection_area(self, wcsim):
+        """
+        Calculate the area of the intersection polygon. If some
+        intersections fail due to a bug/limitation of ``spherical_geometry``
+        then the area of the valid intersections will be returned.
+        If images do not intersect or intersection fails, 0 will be returned.
+        """
+        if isinstance(wcsim, (WCSImageCatalog, RefCatalog)):
+            try:
+                return np.fabs(self.intersection(wcsim).area())
+            except MalformedPolygonError:
+                return 0.0
+
+        else:
+            # this is bug workaround:
+            area = 0.0
+            for wim in wcsim:
+                try:
+                    area += np.fabs(
+                        self.polygon.intersection(wim.polygon).area()
+                    )
+                except MalformedPolygonError:
+                    continue
+            return area
+
     def _calc_chip_bounding_polygon(self, stepsize=None):
         """
         Compute image's bounding polygon.
@@ -517,8 +545,8 @@ class WCSGroupCatalog(object):
                             "'WCSImageCatalog' objects")
 
         self._name = name
-        self.update_bounding_polygon()
         self._catalog = self.create_group_catalog()
+        self.update_bounding_polygon()
 
     @property
     def name(self):
@@ -578,12 +606,31 @@ class WCSGroupCatalog(object):
         """
         return sum(im.intersection_area(wcsim) for im in self._images)
 
+    def _guarded_intersection_area(self, wcsim):
+        """
+        Calculate the area of the intersection polygon. If some
+        intersections fail due to a bug/limitation of ``spherical_geometry``
+        then the area of the valid intersections will be returned.
+        If images do not intersect or intersection fails, 0 will be returned.
+        """
+        return sum(im._guarded_intersection_area(wcsim) for im in self._images)
+
     def update_bounding_polygon(self):
         """ Recompute bounding polygons of the member images.
         """
         polygons = [im.polygon for im in self._images]
         if polygons:
-            self._polygon = SphericalPolygon.multi_union(polygons)
+            try:
+                self._polygon = SphericalPolygon.multi_union(polygons)
+            except MalformedPolygonError:
+                log.warning(
+                    "MalformedPolygonError in spherical_geometry. Using "
+                    "convex hull instead of multi_union. Alignment order "
+                    "may be sub-optimal."
+                )
+                refcat = RefCatalog(self._catalog)
+                self._polygon = refcat.polygon
+
         else:
             self._polygon = SphericalPolygon([])
 
@@ -688,7 +735,7 @@ class WCSGroupCatalog(object):
 
             col_catname = table.MaskedColumn([catname], name='cat_name')
             col_catname = col_catname[[False]]
-            col_imcatidx = table.MaskedColumn([], dtype=np.int,
+            col_imcatidx = table.MaskedColumn([], dtype=int,
                                               name='_imcat_idx')
             col_id = table.MaskedColumn(image.catalog['id'])
             col_x = table.MaskedColumn([], name='x', dtype=np.double)
@@ -748,8 +795,8 @@ class WCSGroupCatalog(object):
 
         Parameters
         ----------
-        tanplane_wcs: ImageGWCS
-            A `ImageGWCS` object that will provide transformations to
+        tanplane_wcs: TPWCS
+            A `TPWCS` object that will provide transformations to
             the tangent plane to which sources of this catalog a should be
             "projected".
 
@@ -864,8 +911,8 @@ class WCSGroupCatalog(object):
         refcat: RefCatalog
             A `RefCatalog` object that contains a catalog of reference sources.
 
-        tanplane_wcs: ImageGWCS
-            A `ImageGWCS` object that will provide transformations to
+        tanplane_wcs: TPWCS
+            A `TPWCS` object that will provide transformations to
             the tangent plane to which sources of this catalog a should be
             "projected".
 
@@ -980,12 +1027,12 @@ class WCSGroupCatalog(object):
         log.info("Computed '{:s}' fit for {}:".format(fitgeom, self.name))
         if fitgeom == 'shift':
             log.info("XSH: {:.6g}  YSH: {:.6g}".format(*fit['shift']))
-        elif fitgeom == 'rscale' and fit['proper']:
+        elif fitgeom in ['rshift', 'rscale'] and fit['proper']:
             log.info(
                 "XSH: {:.6g}  YSH: {:.6g}    ROT: {:.6g}    SCALE: {:.6g}"
                 .format(*fit['shift'], fit['proper_rot'], fit['<scale>'])
             )
-        elif fitgeom == 'general' or (fitgeom == 'rscale' and not
+        elif fitgeom == 'general' or (fitgeom in ['rshift', 'rscale'] and not
                                       fit['proper']):
             log.info("XSH: {:.6g}  YSH: {:.6g}    PROPER ROT: {:.6g}    "
                      .format(*fit['shift'], fit['proper_rot']))
@@ -1005,12 +1052,13 @@ class WCSGroupCatalog(object):
 
         return fit
 
-    def apply_affine_to_wcs(self, tanplane_wcs, matrix, shift, meta=None):
+    @deprecated_renamed_argument('tanplane_wcs', 'ref_tpwcs', since='0.6.5')
+    def apply_affine_to_wcs(self, ref_tpwcs, matrix, shift, meta=None):
         """ Applies a general affine transformation to the WCS. """
         for imcat in self:
-            imcat.tpwcs.set_correction(matrix, shift, ref_tpwcs=tanplane_wcs, meta=meta)
+            imcat.tpwcs.set_correction(matrix, shift, ref_tpwcs=ref_tpwcs, meta=meta)
 
-    def align_to_ref(self, refcat, match=None, minobj=None,
+    def align_to_ref(self, refcat, ref_tpwcs=None, match=None, minobj=None,
                      fitgeom='rscale', nclip=3, sigma=(3.0, 'rmse')):
         """
         Matches sources from the image catalog to the sources in the
@@ -1082,8 +1130,11 @@ class WCSGroupCatalog(object):
         Parameters
         ----------
         refcat: RefCatalog
-            A `RefCatalog` object that contains a catalog of reference sources
-            as well as a valid reference WCS.
+            A `RefCatalog` object that contains a catalog of reference sources.
+
+        ref_tpwcs : TPWCS
+            A `TPWCS` object that defines a projection tangent plane to be
+            used for matching and fitting during alignment.
 
         match: MatchCatalogs, function, None, optional
             A callable that takes two arguments: a reference catalog and an
@@ -1126,7 +1177,6 @@ class WCSGroupCatalog(object):
             This parameter is ignored when ``nclip`` is either `None` or 0
             or when ``match`` is `False`.
 
-
         Returns
         -------
         bool
@@ -1146,18 +1196,13 @@ class WCSGroupCatalog(object):
         for imcat in self:
             imcat.fit_status = "FAILED: Unknown error"
 
-        if minobj is None:
-            if fitgeom == 'general':
-                minobj = 3
-            elif fitgeom == 'rscale':
-                minobj = 2
-            else:
-                minobj = 1
+        minobj = SUPPORTED_FITGEOM_MODES[fitgeom]
 
-        tanplane_wcs = deepcopy(self._images[0].tpwcs)
+        if ref_tpwcs is None:
+            ref_tpwcs = deepcopy(self._images[0].tpwcs)
 
-        self.calc_tanp_xy(tanplane_wcs=tanplane_wcs)
-        refcat.calc_tanp_xy(tanplane_wcs=tanplane_wcs)
+        self.calc_tanp_xy(tanplane_wcs=ref_tpwcs)
+        refcat.calc_tanp_xy(tanplane_wcs=ref_tpwcs)
 
         nmatches, mref_idx, minput_idx = self.match2ref(
             refcat=refcat,
@@ -1172,7 +1217,7 @@ class WCSGroupCatalog(object):
                 imcat.fit_status = 'FAILED: not enough matches'
             return False
 
-        fit = self.fit2ref(refcat=refcat, tanplane_wcs=tanplane_wcs,
+        fit = self.fit2ref(refcat=refcat, tanplane_wcs=ref_tpwcs,
                            fitgeom=fitgeom, nclip=nclip, sigma=sigma)
 
         fit_info = {
@@ -1204,7 +1249,7 @@ class WCSGroupCatalog(object):
             })
 
         self.apply_affine_to_wcs(
-            tanplane_wcs=tanplane_wcs,
+            ref_tpwcs=ref_tpwcs,
             matrix=fit['matrix'],
             shift=fit['shift'],
             # meta=meta
@@ -1264,7 +1309,7 @@ class RefCatalog(object):
 
     @property
     def name(self):
-        """ Get/set :py:class:`WCSImageCatalog` object's name.
+        """ Get/set :py:class:`RefCatalog` object's name.
         """
         return self._name
 
@@ -1358,6 +1403,31 @@ class RefCatalog(object):
                 )
             return area
 
+    def _guarded_intersection_area(self, wcsim):
+        """
+        Calculate the area of the intersection polygon. If some
+        intersections fail due to a bug/limitation of ``spherical_geometry``
+        then the area of the valid intersections will be returned.
+        If images do not intersect or intersection fails, 0 will be returned.
+        """
+        if isinstance(wcsim, (WCSImageCatalog, RefCatalog)):
+            try:
+                return np.fabs(self.intersection(wcsim).area())
+            except MalformedPolygonError:
+                return 0.0
+
+        else:
+            # this is bug workaround:
+            area = 0.0
+            for wim in wcsim:
+                try:
+                    area += np.fabs(
+                        self.polygon.intersection(wim.polygon).area()
+                    )
+                except MalformedPolygonError:
+                    continue
+            return area
+
     def _calc_cat_convex_hull(self):
         """
         Calculate spherical polygon corresponding to the convex hull bounding
@@ -1507,8 +1577,8 @@ class RefCatalog(object):
 
         Parameters
         ----------
-        tanplane_wcs: ImageGWCS
-            A `ImageGWCS` object that will provide transformations to
+        tanplane_wcs: TPWCS
+            A `TPWCS` object that will provide transformations to
             the tangent plane to which sources of this catalog a should be
             "projected".
 
